@@ -10,6 +10,7 @@
 import struct
 import numpy as np
 import pymech.exadata as exdat
+import sys
 
 from pymech.log import logger
 
@@ -93,7 +94,6 @@ def readnek(fname):
 		elif (v == 'T'):
 			var[3] = 1
 		elif (v == 'S'):
-			logger.warning("Reading scalar variables is an experimental feature")
 			# For example: variables = 'XS44'
 			index_s = variables.index('S')
 			nb_scalars = int(variables[index_s+1:])
@@ -221,30 +221,18 @@ def writenek(fname, data):
 	if (data.var[2] > 0): vars += 'P'
 	if (data.var[3] > 0): vars += 'T'
 	if (data.var[4] > 0):
-		logger.warning("Writing scalar variables is an experimental feature")
 		# TODO: check if header for scalars are written with zeros filled as S01
 		vars += 'S{:02d}'.format(data.var[4])
 	#
 	# get word size
 	if (data.wdsz == 4):
-		realtype = 'f'
+		logger.debug('Writing single-precision file')
 	elif (data.wdsz == 8):
-		realtype = 'd'
+		logger.debug('Writing double-precision file')
 	else:
 		logger.error('Could not interpret real type (wdsz = %i)' % (data.wdsz))
 		return -2
 	#
-	# get endian
-	if (data.endian == 'little'):
-		logger.debug('Writing little-endian file\n')
-		emode = '<'
-	elif (data.endian == 'big'):
-		logger.debug('Writing big-endian file\n')
-		emode = '>'
-	else:
-		logger.error('Could not interpret endianness')
-		return -3
-	#
 	# generate header
 	header = '#std %1i %2i %2i %2i %10i %10i %20.13E %9i %6i %6i %s\n' %(
 		data.wdsz, data.lr1[0], data.lr1[1], data.lr1[2], data.nel, nelf,
@@ -254,13 +242,33 @@ def writenek(fname, data):
 	header = header.ljust(132)
 	outfile.write(header.encode('utf-8'))
 	#
+	# decide endianness
+	byteswap = False
+	if data.endian == 'big':
+		if sys.byteorder == 'little':
+			byteswap = True
+		logger.debug('Writing big-endian file')
+	elif data.endian == 'little':
+		logger.debug('Writing little-endian file')
+		if sys.byteorder == 'big':
+			byteswap = True
+	else:
+		logger.debug('Unrecognized endianness, writing native {:s}-endian file'.format(sys.byteorder))
+	#
+	def correct_endianness(a):
+		''' Return the array with the requested endianness'''
+		if byteswap:
+			return a.byteswap()
+		else:
+			return a
+	#
 	# write tag (to specify endianness)
-	etagb = struct.pack(emode+'f', 6.54321)
-	outfile.write(etagb)
+	endianbytes = np.array([6.54321], dtype=np.float32)
+	correct_endianness(endianbytes).tofile(outfile)
 	#
 	# generate and write element map for the file
-	elmap = range(data.nel+1)[1:]
-	outfile.write(struct.pack(emode+nelf*'i', *elmap))
+	elmap = np.linspace(1, data.nel, data.nel, dtype=np.int32)
+	correct_endianness(elmap).tofile(outfile)
 	#
 	#---------------------------------------------------------------------------
 	# WRITE DATA
@@ -269,49 +277,32 @@ def writenek(fname, data):
 	# compute total number of points per element
 	npel = data.lr1[0] * data.lr1[1] * data.lr1[2]
 	#
+	def write_ndarray_to_file(a):
+		'''Write a data array to the output file in the requested precision and endianness'''
+		if data.wdsz == 4:
+			correct_endianness(a.astype(np.float32)).tofile(outfile)
+		else:
+			correct_endianness(a).tofile(outfile)
+	#
 	# write geometry
 	for iel in elmap:
-		fo = np.zeros(npel)
 		for idim in range(data.var[0]): # if var[0] == 0, geometry is not written
-			ip = 0
-			for iz in range(data.lr1[2]):
-				for iy in range(data.lr1[1]):
-					fo[ip:ip+data.lr1[0]] = data.elem[iel-1].pos[idim,iz,iy,:]
-					ip += data.lr1[0]
-			outfile.write(struct.pack(emode+npel*realtype, *fo))
+			write_ndarray_to_file(data.elem[iel-1].pos[idim, :, :, :])
 	#
 	# write velocity
 	for iel in elmap:
-		fo = np.zeros(npel)
 		for idim in range(data.var[1]): # if var[1] == 0, velocity is not written
-			ip = 0
-			for iz in range(data.lr1[2]):
-				for iy in range(data.lr1[1]):
-					fo[ip:ip+data.lr1[0]] = data.elem[iel-1].vel[idim,iz,iy,:]
-					ip += data.lr1[0]
-			outfile.write(struct.pack(emode+npel*realtype, *fo))
+			write_ndarray_to_file(data.elem[iel-1].vel[idim, :, :, :])
 	#
 	# write pressure
 	for iel in elmap:
-		fo = np.zeros(npel)
 		for ivar in range(data.var[2]): # if var[2] == 0, pressure is not written
-			ip = 0
-			for iz in range(data.lr1[2]):
-				for iy in range(data.lr1[1]):
-					fo[ip:ip+data.lr1[0]] = data.elem[iel-1].pres[ivar,iz,iy,:]
-					ip += data.lr1[0]
-			outfile.write(struct.pack(emode+npel*realtype, *fo))
+			write_ndarray_to_file(data.elem[iel-1].pres[ivar, :, :, :])
 	#
 	# write temperature
 	for iel in elmap:
-		fo = np.zeros(npel)
 		for ivar in range(data.var[3]): # if var[3] == 0, temperature is not written
-			ip = 0
-			for iz in range(data.lr1[2]):
-				for iy in range(data.lr1[1]):
-					fo[ip:ip+data.lr1[0]] = data.elem[iel-1].temp[ivar,iz,iy,:]
-					ip += data.lr1[0]
-			outfile.write(struct.pack(emode+npel*realtype, *fo))
+			write_ndarray_to_file(data.elem[iel-1].temp[ivar, :, :, :])
 	#
 	# write scalars
 	#
@@ -320,32 +311,25 @@ def writenek(fname, data):
 	# are in the inner loop
 	#
 	for ivar in range(data.var[4]): # if var[4] == 0, scalars are not written
-		fo = np.zeros(npel)
 		for iel in elmap:
-			ip = 0
-			for iz in range(data.lr1[2]):
-				for iy in range(data.lr1[1]):
-					fo[ip:ip+data.lr1[0]] = data.elem[iel-1].scal[ivar,iz,iy,:]
-					ip += data.lr1[0]
-			outfile.write(struct.pack(emode+npel*realtype, *fo))
-	#
+			write_ndarray_to_file(data.elem[iel-1].scal[ivar, :, :, :])
 	#
 	# write max and min of every field in every element (forced to single precision)
 	if (data.ndim==3):
 		#
 		for iel in elmap:
 			for idim in range(data.var[0]):
-				outfile.write(struct.pack(emode+'f', np.min(data.elem[iel-1].pos[idim, :,:,:])))
-				outfile.write(struct.pack(emode+'f', np.max(data.elem[iel-1].pos[idim, :,:,:])))
+				correct_endianness(np.min(data.elem[iel-1].pos[idim, :,:,:]).astype(np.float32)).tofile(outfile)
+				correct_endianness(np.max(data.elem[iel-1].pos[idim, :,:,:]).astype(np.float32)).tofile(outfile)
 			for idim in range(data.var[1]):
-				outfile.write(struct.pack(emode+'f', np.min(data.elem[iel-1].vel[idim, :,:,:])))
-				outfile.write(struct.pack(emode+'f', np.max(data.elem[iel-1].vel[idim, :,:,:])))
+				correct_endianness(np.min(data.elem[iel-1].vel[idim, :,:,:]).astype(np.float32)).tofile(outfile)
+				correct_endianness(np.max(data.elem[iel-1].vel[idim, :,:,:]).astype(np.float32)).tofile(outfile)
 			for idim in range(data.var[2]):
-				outfile.write(struct.pack(emode+'f', np.min(data.elem[iel-1].pres[idim, :,:,:])))
-				outfile.write(struct.pack(emode+'f', np.max(data.elem[iel-1].pres[idim, :,:,:])))
+				correct_endianness(np.min(data.elem[iel-1].pres[idim, :,:,:]).astype(np.float32)).tofile(outfile)
+				correct_endianness(np.max(data.elem[iel-1].pres[idim, :,:,:]).astype(np.float32)).tofile(outfile)
 			for idim in range(data.var[3]):
-				outfile.write(struct.pack(emode+'f', np.min(data.elem[iel-1].temp[idim, :,:,:])))
-				outfile.write(struct.pack(emode+'f', np.max(data.elem[iel-1].temp[idim, :,:,:])))
+				correct_endianness(np.min(data.elem[iel-1].temp[idim, :,:,:]).astype(np.float32)).tofile(outfile)
+				correct_endianness(np.max(data.elem[iel-1].temp[idim, :,:,:]).astype(np.float32)).tofile(outfile)
 
 	# close file
 	outfile.close()
